﻿using System;
using System.Collections.Generic;
using System.Drawing;
 
using System.Text;

namespace LifeGameScreenSaver
{
    
    public class World
    {

        public int WorldHeight { get; set; }

        public int WorldWidth { get; set; }

        public Life[,] LifeBounds;
      
        public World(int I_WorldWidth, int I_WorldHeight)
        {
            WorldWidth = I_WorldWidth;
            WorldHeight = I_WorldHeight;
           
            LifeBounds = new Life[I_WorldWidth, I_WorldHeight];

            Random curRandom = new Random();
            //初始化世界
            for (int x = 0; x < I_WorldHeight; x++)
            {
                for (int y = 0; y < I_WorldWidth; y++)
                {
                    if (curRandom.Next(0, 2) == 0)
                    {

                        LifeBounds[x, y] = new Life(true, x, y);
                    }
                    else
                    {
                        LifeBounds[x, y] = new Life(false, x, y);
                    }
                   
                }
            }
        }
       
        public void RefreshWorld()
        {
            foreach (var curLife in LifeBounds)
            {
                int liveCount = 0;

                int x = curLife.X;
                int y = curLife.Y;
                if (x > 0 && y > 0)
                    if (LifeBounds[x - 1, y - 1].CurLiveStatus) liveCount++;//左上角
                if (x > 0)
                    if (LifeBounds[x - 1, y].CurLiveStatus) liveCount++;//左边
                if (x > 0 && y < WorldHeight - 1)
                    if (LifeBounds[x - 1, y + 1].CurLiveStatus) liveCount++;//左下角

                if (y > 0)
                    if (LifeBounds[x, y - 1].CurLiveStatus) liveCount++;//上方
                if (y < WorldHeight - 1)
                    if (LifeBounds[x, y + 1].CurLiveStatus) liveCount++;//下方

                if (x < WorldWidth - 1 && y > 0)
                    if (LifeBounds[x + 1, y - 1].CurLiveStatus) liveCount++;//右上角
                if (x < WorldWidth - 1)
                    if (LifeBounds[x + 1, y].CurLiveStatus) liveCount++;//右方
                if (x < WorldWidth - 1 && y < WorldHeight - 1)
                    if (LifeBounds[x + 1, y + 1].CurLiveStatus) liveCount++;//右下角

                if (liveCount == 3)
                {
                    //周围有三个活，则活（以前的状态不论）
                    curLife.NextLiveStatus = true;
                }
                else if (liveCount == 2)
                {
                    curLife.NextLiveStatus = curLife.CurLiveStatus;
                }
                else if (liveCount < 2 || liveCount > 3)
                {
                    //周围大于三个，则死
                    curLife.NextLiveStatus = false;
                }

            }
            foreach (var curLife in LifeBounds)
            {
                curLife.RefreshStatus();
            }

        }



        public Bitmap GetWorldBitmap(int I_LifeWidth, Color I_LiveColor, Color I_DeadColor,int I_TargetWidth,int I_TargetHeight)
        {
            int drawWidth = I_TargetWidth / I_LifeWidth + 1;
            int drawHeight = I_TargetHeight / I_LifeWidth + 1;

            int minX = (this.WorldWidth - drawWidth) / 2;
            int maxX = minX + drawWidth;

            int minY = (this.WorldHeight - drawHeight) / 2;
            int maxY = minY + drawHeight;

 
            Bitmap R_Result = new Bitmap(I_TargetWidth, I_TargetHeight);

            
            using (Graphics e = Graphics.FromImage(R_Result))
            {
                using (SolidBrush curBrush = new SolidBrush(I_LiveColor))
                {
                    using (SolidBrush textBrush = new SolidBrush(Color.Red))
                    {
                        foreach (var curLife in LifeBounds)
                        {

                            if (curLife.X >= minX && curLife.X < maxX
                                && curLife.Y >= minY && curLife.Y < maxY)
                            {
                                if (curLife.CurLiveStatus)
                                {
                                    curBrush.Color = I_LiveColor;
                                }
                                else
                                {
                                    curBrush.Color = I_DeadColor;
                                }

                                e.FillRectangle(curBrush, (curLife.X - minX) * I_LifeWidth, (curLife.Y - minY) * I_LifeWidth, I_LifeWidth, I_LifeWidth);
                            }

                        }

                        e.DrawString(GetLivePercent().ToString("F4"), new Font("宋体", 20, GraphicsUnit.Pixel), textBrush, new PointF(20, 20));
                    }
                }
               
            }

            

            return R_Result;
        }


        public float GetLivePercent()
        {
            float R_Result;
            int liveCount=0;
            int totalCount=0;

            foreach (var curLife in LifeBounds)
            {
                if(curLife.CurLiveStatus)
                {
                    liveCount++;
                }
                totalCount++;
            }
            R_Result = ((float)liveCount) /( (float)totalCount);
            return R_Result;
        }
    }
}
